#!/usr/bin/env python3

import asyncio
import sys
from typing import Any, Dict, List
from mcp.server.models import InitializationOptions
from mcp.server import NotificationOptions, Server
from mcp.server.stdio import stdio_server
from mcp.types import (
    CallToolRequest,
    CallToolResult,
    ListToolsRequest,
    ListToolsResult,
    Tool,
    TextContent,
)

server = Server("math-mcp-server")

@server.list_tools()
async def handle_list_tools() -> List[Tool]:
    return [
        Tool(
            name="calculate",
            description="Perform mathematical calculations",
            inputSchema={
                "type": "object",
                "properties": {
                    "expression": {
                        "type": "string",
                        "description": "Mathematical expression to evaluate (e.g., '2 + 3 * 4')",
                    }
                },
                "required": ["expression"],
            },
        ),
        Tool(
            name="solve_equation",
            description="Solve linear equations",
            inputSchema={
                "type": "object",
                "properties": {
                    "equation": {
                        "type": "string",
                        "description": "Linear equation to solve (e.g., '2x + 5 = 15')",
                    }
                },
                "required": ["equation"],
            },
        ),
    ]

@server.call_tool()
async def handle_call_tool(name: str, arguments: dict) -> List[TextContent]:
    if name == "calculate":
        expression = arguments.get("expression", "")
        try:
            result = safe_eval(expression)
            return [TextContent(type="text", text=f"{expression} = {result}")]
        except Exception as e:
            return [TextContent(type="text", text=f"Error: {str(e)}")]
    
    elif name == "solve_equation":
        equation = arguments.get("equation", "")
        try:
            solution = solve_linear_equation(equation)
            return [TextContent(type="text", text=f"Solution: {solution}")]
        except Exception as e:
            return [TextContent(type="text", text=f"Error: {str(e)}")]
    
    else:
        raise ValueError(f"Unknown tool: {name}")



def safe_eval(expression: str) -> float:
    """Safely evaluate mathematical expressions"""
    import re
    
    # Remove whitespace
    cleaned = re.sub(r'\s+', '', expression)
    
    # Only allow numbers, operators, parentheses, and decimal points
    if not re.match(r'^[0-9+\-*/().]+$', cleaned):
        raise ValueError("Invalid characters in expression")
    
    # Use eval with restricted globals
    try:
        return eval(cleaned, {"__builtins__": {}})
    except Exception:
        raise ValueError("Invalid mathematical expression")

def solve_linear_equation(equation: str) -> str:
    """Solve linear equations in format ax + b = c"""
    import re
    
    parts = equation.split('=')
    if len(parts) != 2:
        raise ValueError("Equation must have exactly one equals sign")
    
    left = parts[0].strip()
    right = float(parts[1].strip())
    
    # Parse ax + b format
    match = re.match(r'^(-?\d*\.?\d*)x\s*([+-]\s*\d+\.?\d*)?$', left)
    if not match:
        raise ValueError("Only linear equations in format 'ax + b = c' are supported")
    
    a = float(match.group(1) or '1')
    b = float(match.group(2).replace(' ', '') if match.group(2) else '0')
    
    if a == 0:
        raise ValueError("Coefficient of x cannot be zero")
    
    x = (right - b) / a
    return f"x = {x}"

async def main():
    async with stdio_server(server) as (read_stream, write_stream):
        await server.run(
            read_stream, write_stream, InitializationOptions(
                server_name="math-mcp-server",
                server_version="1.0.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            )
        )

def main():
    asyncio.run(main())

if __name__ == "__main__":
    main()